#pragma once
#include <algorithm>
#include <string>

// Expand a macro and convert to a (C-style) string, eg xstr( NUMDUMMYPARAMS ) -> "16"
#define xstr( expr ) CSTRINGIFY( expr )
#define CSTRINGIFY( expr ) #expr

// STRINGIFY macro to make multiline string constants (eg vertex shader code) easier to write. Note: comments will be stripped
#define STRINGIFY( expr ) std::string( #expr )

namespace utils
{
// Clamp the value to not be less than low and not greater than high.
// Result will be low if high < low.
inline float clamp( float value, float low, float high )
{
	return std::max( std::min( value, high ), low );
}

// Remap a value from one range [low,high] to another range [newLow,newHigh].
inline float map( float value, float low, float high, float newLow, float newHigh )
{
	float res = newLow + ( value - low ) * ( newHigh - newLow ) / ( high - low );
	return clamp( res, newLow, newHigh );
}

// Fill a buffer (up to the size of the buffer) with a string, and null/zero-fill any space if the string is shorter than the buffer
inline void StrToBuff( const std::string& str, char* buff, size_t buffSize )
{
	size_t copyLen = str.size();
	if( copyLen > buffSize )
		copyLen = buffSize;

	memcpy( buff, str.c_str(), copyLen );

	if( copyLen < buffSize )
	{
		memset( buff + copyLen, 0, buffSize - copyLen );
	}
}

}//End namespace utils
